extern crate clap;

use std::error::Error;
use std::fs::File;
use std::io::{BufReader, Read};
use clap::{arg_enum, clap_app, value_t};

arg_enum! {
    #[derive(Debug)]
    enum DaysImplemented {
        Day1,
    }
}

fn main() -> Result<(), Box<dyn Error>> {
    let matches = clap_app!(AdventOfCode2021 =>
        (author: "Anthony Cicchetti")
        (@setting ArgRequiredElseHelp)
        (@arg day_to_execute: -d --day * +takes_value possible_values(&DaysImplemented::variants()))
        (@arg part: -p --part * +takes_value possible_values(&["1", "2"]))
        (@arg input_file: -f --file {file_exists} * conflicts_with[input_string] +takes_value "Input file")
        (@arg input_string: required_unless[input_file] "Input string")
    ).get_matches();
    let day: DaysImplemented =
        value_t!(matches.value_of("day_to_execute"), DaysImplemented).unwrap(); // Safe unwrap because value is required
    let part = value_t!(matches.value_of("part"), u8).unwrap();
    let input = match matches.value_of("input_file") {
        None => value_t!(matches.value_of("input_string"), String).unwrap(),
        Some(path) => {
            let mut s = String::new();
            let _ = BufReader::new(File::open(path)?).read_to_string(&mut s);
            s
        }
    };

    println!("Executing {}, Part {}", day, part);
    match (day, part) {
        (DaysImplemented::Day1, 1) => println!("{}", day_01::part1(&input)),
        (DaysImplemented::Day1, 2) => println!("{}", day_01::part2(&input)),
        _ => {unimplemented!()}
    };

    Ok(())
}

fn file_exists(val: String) -> Result<(), String> {
    if std::fs::metadata(&val).is_ok() {
        Ok(())
    } else {
        Err(String::from(format!("File at {} doesn't exist", &val)))
    }
}
